/* Copyright (C) 2000,2001,2002 Manuel Amador (Rudd-O)
   This file is part of Directory administrator.

   Directory administrator is free software; you can redistribute it 
   and/or modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   Directory administrator is distributed in the hope that it will be 
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Directory administrator; if not, send e-mail to amador@alomega.com 
*/


#include "appsupport.h"
#include "profile.h"
#include "users.h"
#include "groups.h"
#include "appglobals.h"
#include "appfunctions.h"
#include "profile.h"
#include "newdruids.h"
#include "usergrouplists.h"
#include "ldaptransaction.h"
#include "dir_entry.h"

gboolean
newgroup_commit (GnomeDruidPage * gnomedruidpage)
{
  ldaptransaction*t;
  diradmin_group *newuser = NULL;
  int ldap_errors = LDAP_SUCCESS;
  GtkWidget *window = NULL;
  gchar *dn = NULL;
  gchar *basedn = NULL;
  gchar *gid = NULL;
  dir_entry *dentry = NULL;

  if (connection_profile_is_connected (current_connection_profile) == FALSE)
    return ERROR;

  window = lookup_widget (GTK_WIDGET (gnomedruidpage), "druid_new_group");
  basedn = gtk_entry_get_text (GTK_ENTRY (lookup_widget (window, "baseou")));
  //this is a stupid kludge to avoid constructing the DN, for the time being
  if (GTK_TOGGLE_BUTTON
      (lookup_widget (GTK_WIDGET (gnomedruidpage), "automaticgid"))->active)
    gid = get_lowest_gid (current_connection_profile);
  else
    gid =
      g_strdup (gtk_entry_get_text
		(GTK_ENTRY (lookup_widget (window, "gidnumber"))));
  dn =
    g_strconcat ("cn=",
		 gtk_entry_get_text (GTK_ENTRY
				     (lookup_widget (window, "cn"))), ",",
		 basedn, NULL);

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (window, "gidnumber")), gid);

  newuser = create_group_struct_from_dialogbox (current_connection_profile,
						window, dn);

  t = diradmin_group_create_ldapdiff (newuser);
      ldap_errors =
	connection_profile_create_record (current_connection_profile,
					  t->mods, dn);
  ldaptransaction_destroy(t);
  if (ldap_errors)
    {
      if (ldap_errors == LDAP_UNDEFINED_TYPE)
	gtk_widget_show (create_messagebox_with_message
			 (
"The group could not be created because your directory hasn't\nsupport for certain features.  Go to Settings->Preferences and\nset the appropriate compatibility options."));
      else
	gtk_widget_show (create_messagebox_with_message
			 (ldap_err2string (ldap_errors)));
    }
  else
    {
      dentry = dir_entry_group_new(
          g_strdup(dn),
          g_strdup(diradmin_group_get_attribute (newuser, "cn")),
          atoi(diradmin_group_get_attribute (newuser, "gidnumber"))
         );

  diradmin_group_destroy (newuser);

//  g_print("\nThe following entry was just added to the directory cache");
//  dir_entry_dump(dentry);

  app_add_dir_entry(dentry);



    }

  g_free (dn);
  g_free (gid);
// TEMP DISABLED  diradmin_group_destroy (newuser);
  if (ldap_errors)
    return ERROR;
  connection_profile_set_last_group_ou
    (connection_profile_list_getbyname
     (connection_profile_list,
      connection_profile_get_name (current_connection_profile)), basedn);
  return OK;
}

gboolean
newuser_commit (GnomeDruidPage * gnomedruidpage)
{
  ldaptransaction * t;
  diradmin_user *newuser = NULL;
  int ldap_errors = LDAP_SUCCESS;
  GtkWidget *window = NULL;
  gchar *dn = NULL;
  gchar *basedn = NULL;
  gchar *uid = NULL;
  dir_entry*dentry=NULL;

  if (connection_profile_is_connected (current_connection_profile) == FALSE)
    return ERROR;
  window = lookup_widget (GTK_WIDGET (gnomedruidpage), "druid_new_user");
  basedn = gtk_entry_get_text (GTK_ENTRY (lookup_widget (window, "baseou")));
  if (GTK_TOGGLE_BUTTON
      (lookup_widget (GTK_WIDGET (gnomedruidpage), "automaticuid"))->active)
    uid = get_lowest_uid (current_connection_profile);
  else
    uid =
      g_strdup (gtk_entry_get_text
		(GTK_ENTRY (lookup_widget (window, "uidnumber"))));
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (window, "uidnumber")), uid);
  dn =
    g_strconcat ("uid=",
		 gtk_entry_get_text (GTK_ENTRY
				     (lookup_widget
				      (window, "uid"))), ",", basedn, NULL);
  g_print ("Base DN for user creation: %s\n", basedn);
  newuser =
    create_user_struct_from_dialogbox
    (current_connection_profile, window, dn);
  /* En esta parte, activamos que el usuario creado con el druida
     sea logon to all servers */
/*  diradmin_user_allowallservers(newuser);*/
  t = diradmin_user_create_ldapdiff (newuser);
      ldap_errors =
	connection_profile_create_record
	(current_connection_profile, t->mods, dn);
  ldaptransaction_destroy(t);

  if (ldap_errors)
    {
      if (ldap_errors == LDAP_UNDEFINED_TYPE)
	gtk_widget_show (create_messagebox_with_message
			 (
"The user could not be created because your directory hasn't\nsupport for certain features.  Go to Settings...Preferences and\nset the appropriate compatibility options"));
      else
	gtk_widget_show
	  (create_messagebox_with_message (ldap_err2string (ldap_errors)));
    }
  else
    {

    dentry=dir_entry_user_new(g_strdup(dn),
g_strdup(diradmin_user_get_attribute (newuser, "uid")),
g_strdup(diradmin_user_get_attribute(newuser, "cn")),
atoi(diradmin_user_get_attribute(newuser, "uidnumber")),
atoi(diradmin_user_get_attribute(newuser, "gidnumber"))
);
//  g_print("\nThe following entry was just added to the directory cache");
//  dir_entry_dump(dentry);

  app_add_dir_entry(dentry);
    }

  g_free (dn);
  g_free (uid);
  diradmin_user_destroy (newuser);
  if (ldap_errors)
    return ERROR;
  connection_profile_set_last_user_ou
    (connection_profile_list_getbyname
     (connection_profile_list,
      connection_profile_get_name (current_connection_profile)), basedn);
  return OK;
}
